# Copyright (C) 1999 Jay Beale
# assisted by Peter Watkins
# Licensed under the GNU General Public License

package Bastille::IPCHAINS;

use Bastille::IO;
@ENV="";
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";

# @ipchainsOptions contains information about the variables this script sets the value of
# in the final firewall script. Each item in @ipchainsOptions is a hash describing a certain
# environment variable. 
#
# Each such hash should have the following name-value pairs:
#	heading		short name for what the variable controls
#	varname		the actual variable name for the /bin/sh firewall script
#	default		the default/suggested value for that variable
#	stanza		where in the /bin/sh script the variable is set
#	type		whether the option is "basic" (all users should be asked about it),
#			or "server", meaning only those who said they need advanced options
#			should be asked about this item. (If a user is only shown "basic"
#			options, this script will quietly set the variable to the default
#			specified in @ipchainsOptions)
#	basic_info	basic explanatory text shown to all users
#	verbose_info	more detailed text shown to those who asked for more verbose explanations

@ipchainsOptions = (

	{
	'heading' => "DNS servers",
	'varname' => "DNS_SERVERS",
	'default' => "0.0.0.0/0",
	'stanza' => "0",
	'type' => "basic",
	'basic_info' => "This controls what external servers you can use for DNS lookups. For regular workstations, this should contain all your nameserver addresses, separated by spaces. If you want to run a caching nameserver and/or run your own DNS, leave this at \"0.0.0.0/0\" so you can query any DNS server. If you set this to an empty value, the firewall script will read the current nameservers from /etc/resolv.conf when it is run, which is the recommended configuration. This default is designed to ensure functionality.",
	'verbose_info' => "DNS servers are used to translate names like \"example.org\" into addresses like \"10.1.2.3\". You need to configure DNS for many pieces of software to function properly. Your system administrator or Internet Service Provider should be able to provide you with this information. Most users should simply leave this at \"0.0.0.0/0\" (or make it blank) so the firewall script will be more forgiving (or do the right thing automatically). For instance, DHCP clients often re-write /etc/resolv.conf when obtaining a new lease. (This means you may want to configure your system to run the firewall script both before _and_ after setting up your DHCP-configured interface if you set this to the safest value, an empty string.) ",
	},

	{
	'heading' => "Trusted interfaces",
	'varname' => "TRUSTED_IFACES",
	'default' => "lo",
	'stanza' => "1",
	'type' => "server",
	'basic_info' => "List the interface names of all interfaces you want to have unrestricted access to this machine. You should at least trust \"lo\", the \"loopback\" interface.",
	'verbose_info' => "Interface names normally look like \"eth0\" for the first Ethernet card, \"ppp0\" for a PPP connection, etc. Any traffic coming from the interfaces listed here will be allowed by the kernel (though TCP Wrappers or the application itself may end up denying the connection attempt). Basically, you will have no kernel-level firewall protecting you from traffic on these interfaces, and should therefore think carefully before changing the default. ",
	},

	{
	'heading' => "Public interfaces",
	'varname' => "PUBLIC_IFACES",
	'default' => "eth+ ppp+ slip+",
	'stanza' => "1",
	'type' => "basic",
	'basic_info' => "List names of all interfaces connected to public/untrusted networks. The \"+\" character is a wildcard, e.g. \"ppp+\" matches any interface name beginning with \"ppp\" in case you have multiple dialup profiles.",
	'verbose_info' => "Using the \"+\" suffix allows you to configure more interfaces (for instance, more PPP dialup entries) without having to modify the firewall script. ",
	},

	{
	'heading' => "Internal (partially trusted) interfaces",
	'varname' => "INTERNAL_IFACES",
	'default' => "",
	'stanza' => "1",
	'type' => "server",
	'basic_info' => "This is for servers that will act as NAT / IP Masq firewalls between local, but not fully trusted, networks and public networks like the Internet. List names of all \"internal\" interfaces that might have full ability to use NAT / IP Masq to contact public networks, but only limited access to services running on this machine. Do not use \"+\" characters; name each interface explicitly.",
	'verbose_info' => "Normal workstations should leave this as the empty default. ",
	},

	{
	'heading' => "TCP services to audit",
	'varname' => "TCP_AUDIT_SERVICES",
	'default' => "telnet ftp imap pop-3 finger sunrpc exec login linuxconf ssh",
	'stanza' => "2",
	'type' => "basic",
	'basic_info' => "List any TCP-based services (name or port number) that you want the kernel to log connection attempts from the \"public\" interfaces.",
	'verbose_info' => "If you have \"syslog\" configured to log \"kern\" messages of \"info\" level, the kernel will automatically log connection attempts from the \"public\" interfaces (only the \"public\" interfaces) to these ports and/or services. This is useful to spot possible probes or attacks. The default setting records connection attempts to several services, although you may not have them installed or enabled. ",
	},

	{
	'heading' => "UDP services to audit",
	'varname' => "UDP_AUDIT_SERVICES",
	'default' => "31337",
	'stanza' => "2",
	'type' => "basic",
	'basic_info' => "As above, but TCP services. Port 31337 was the default port for the infamous \"Back Orifice\" trojan/remote-control app for Windows systems.",
	'verbose_info' => "While attackers probing for Back Orifice may not pose a threat to your Linux system, logging their attempts helps identify the \"bad guys\" ",
	},

	{
	'heading' => "ICMP packet types to audit",
	'varname' => "ICMP_AUDIT_TYPES",
	'default' => "",
	'stanza' => "2",
	'type' => "basic",
	'basic_info' => "As above, but 1) ICMP protocol and 2) these should be specified as types, not numbers. One example is \"echo-request\" which is used by Microsoft ping and tracert [sic] clients.",
	'verbose_info' => "",
	},

	{
	'heading' => "TCP services to make public",
	'varname' => "TCP_PUBLIC_SERVICES",
	'default' => "",
	'stanza' => "3",
	'type' => "basic",
	'basic_info' => "Names or port numbers on which to accept connection attempts from the \"public\" interfaces. Typical workstations will not want to make any services available, though admins may want to enable something like SSH (default port: 22) for remote administration. Those running caching or \"real\" DNS servers on this machine will want to enable domain (or port 53). If you want to make FTP available to clients on the \"public\" interfaces, you will want to allow the range of IP addresses used for \"passive\" FTP connections.",
	'verbose_info' => "You will need to list the names or port numbers of any services running on this machine that you want hosts on the \"public\" network to access. For instance, if you have a local Web server you want to share, add \"80\" for the normal HTTP port. Not doing so means you will be able to access the service locally, but \"public\" hosts will not. ",
	},

	{
	'heading' => "UDP services to make public",
	'varname' => "UDP_PUBLIC_SERVICES",
	'default' => "",
	'stanza' => "3",
	'type' => "basic",
	'basic_info' => "As above. If you're running caching or real DNS servers, you  will need to enable domain (port 53).",
	'verbose_info' => "",
	},

	{
	'heading' => "TCP services to make available to \"internal\" hosts",
	'varname' => "TCP_INTERNAL_SERVICES",
	'default' => "",
	'stanza' => "3",
	'type' => "server",
	'basic_info' => "As above. Note that the \"public\" services will not be made available to \"internal\" hosts unless you also specify those services again here. If you want to make FTP available to clients on the \"internal\" interfaces, you will want to allow the range of IP addresses used for \"passive\" FTP connections. ",
	'verbose_info' => "For instance, a corporate firewall/mailserver might have \"smtp\" enabled on  the public side to accept outside mail, and for \"internal\" interfaces it might allow both \"smtp\" and \"imap\" so local users can both send and get mail; in that case you would set this value to \"smtp imap\". This does not affect IP Masquerading's ability to let masq'ed users access any services on outside/Internet hosts. ",
	},

	{
	'heading' => "UDP services to make available to \"internal\" hosts",
	'varname' => "UDP_INTERNAL_SERVICES",
	'default' => "",
	'stanza' => "3",
	'type' => "server",
	'basic_info' => "As above. Note that the \"public\" services will not be made available to \"internal\" hosts unless you also specify those services again here.",
	'verbose_info' => "As with internal TCP. You do not need to enable domain service if the internal clients are using IP Masq to query outside DNS servers. ",
	},

	{
	'heading' => "Force FTP clients to use \"passive\" FTP",
	'varname' => "FORCE_PASV_FTP",
	'default' => "N",
	'stanza' => "4",
	'type' => "basic",
	'basic_info' => "This has nothing to do with whether you are running an FTP _server_ on this machine; this has to do with how clients running on this machine will talk to _other_ machines running FTP servers reachable through the \"public\" interfaces. By forcing your local FTP clients to use \"passive\" mode, you will not have to be as cautious about blocking specific \"high\" TCP services. Set to \"Y\" to force \"passive\" FTP; the default \"N\" will allow you to use normal, \"active\" FTP. Forcing passive mode (\"Y\") is recommended, but less convenient.",
	'verbose_info' => "Forcing passive FTP will make using some FTP clients more of a hassle, as you may need to manually tell them to use passive mode, but many clients such as Netscape Navigator have no problem with passive FTP. If you have problems with FTP, this is the first place to look. ",
	},

	{
	'heading' => "TCP services to block",
	'varname' => "TCP_BLOCKED_SERVICES",
	'default' => "1024 2049 2065:2090 6000:6020 7100",
	'stanza' => "5",
	'type' => "basic",
	'basic_info' => "These rules take effect _after_ the TCP services to make public. If you allow the use of \"active\" FTP clients (FORCE_PASV_FTP at its efault of \"0\"), you will need to be careful here, and will want to make sure you block all TCP services listening on high ports. If you are forcing \"passive\" FTP, you may ignore this setting.",
	'verbose_info' => "We have listed the services we have observed. To be more cautious, you should look at the output of 'lsof -i' (run as root) once the system is up and all services are running. ",
	},

	{
	'heading' => "UDP services to block",
	'varname' => "UDP_BLOCKED_SERVICES",
	'default' => "1066 2049 6770",
	'stanza' => "5",
	'type' => "basic",
	'basic_info' => "As above, the UDP services to make public will take precedence. The high UDP services that you do not block will be reachable by any allowed NTP or DNS server. Sites with more such \"high UDP\" services, or global DNS availability (as is the default, DNS_SERVERS=\"0.0.0.0/0\"), will want to be sure they have all such high UDP services listed.",
	'verbose_info' => "",
	},

	{
	'heading' => "ICMP allowed types",
	'varname' => "ICMP_ALLOWED_TYPES",
	'default' => "destination-unreachable echo-reply time-exceeded",
	'stanza' => "5",
	'type' => "basic",
	'basic_info' => "The default suggestion allows you to probe other hosts with ping and traceroute. Minimally you will need to allow \"destination-unreachable\".",
	'verbose_info' => "\"desitination-unreachable\" lets other machines' servers tell your system when things aren't right; don't disable this unless you really know what you're getting into. If you don't allow \"echo-reply\" and \"time-exceeded\", you won't be able to use ping and traceroute to debug issues on the \"public\" networks. ",
	},

	{
	'heading' => "Source address verification",
	'varname' => "ENABLE_SRC_ADDR_VERIFY",
	'default' => "Y",
	'stanza' => "6",
	'type' => "basic",
	'basic_info' => "This configures the kernel to block traffic likely to have spoofed IP addresses. Set to \"N\" to disable. The default (\"Y\") is highly recommended.",
	'verbose_info' => "This is a standard, and highly recommended, precaution. ",
	},

	{
	'heading' => "IP masquerading (NAT)",
	'varname' => "IP_MASQ_NETWORK",
	'default' => "",
	'stanza' => "7",
	'type' => "server",
	'basic_info' => "If this machine will be used as an IP Masquerading / Network Address Translation gateway, enter the networks to be masqueraded (from trusted interfaces). Example: \"10.0.0.0\". If you will not be using IP Masq / NAT, leave this as the empty default.",
	'verbose_info' => "Note this expects _network_ addresses (either with 0's on the end or with explicit netmasks), _not_ interface names. ",
	},

	{
	'heading' => "IP masquerading kernel modules",
	'varname' => "IP_MASQ_MODULES",
	'default' => "ftp raudio vdolive",
	'stanza' => "7",
	'type' => "server",
	'basic_info' => "Special kernel modules are required to provide certain services via IP Masquerading. Possible modules include cuseeme, ftp, irc, quake, raudio, and vdolive. The script assumes each name should have the usual prefix, e.g. \"raudio\" will cause the script to load the \"ip_masq_raudio\" module.",
	'verbose_info' => "",
	},

	{
	'heading' => "Reject method",
	'varname' => "REJECT_METHOD",
	'default' => "DENY",
	'stanza' => "8",
	'type' => "basic",
	'basic_info' => "How the kernel rejects blocked traffic. \"REJECT\" is friendly, lets the remote host know you're blocking their atttempt (and can therefore be used to prove you're on the network). \"DENY\" is unfriendly, simply drops the connection attempt, leaving the remote host to wait, and probably give up after some time.",
	'verbose_info' => "There's no definite right answer here. You will probably not be _completely_ invisible, even if you choose \"DENY\", but with \"DENY\" and _no_ public services, you will not be visible to casual probes. ",
	},

	{
	'heading' => "Interfaces for DHCP queries",
	'varname' => "DHCP_IFACES",
	'default' => "",
	'stanza' => "9",
	'type' => "basic",
	'basic_info' => "List the names of any interfaces this machine will need to make DHCP _queries_ on to configure _its own_ interfaces. For example, a cable modem user with a single ethernet interface would probably wnat to set this to \"eth0\".",
	'verbose_info' => "",
	},

	{
	'heading' => "NTP servers",
	'varname' => "NTP_SERVERS",
	'default' => "",
	'stanza' => "10",
	'type' => "basic",
	'basic_info' => "If you want to queries NTP time servers to synchronize your system time, enter IP addresses or networks for those servers here. If you don't intend to make NTP queries, leave this as the empty default.",
	'verbose_info' => "The same warnings about blocked UDP services and DNS servers apply here; the hosts and networks you list here can connect to any high UDP port not explicitly blocked. ",
	},

	{
	'heading' => "ICMP types to disallow _outbound_",
	'varname' => "ICMP_OUTBOUND_DISABLED_TYPES",
	'default' => "destination-unreachable time-exceeded",
	'stanza' => "11",
	'type' => "basic",
	'basic_info' => "By disabling this default list of ICMP types, your machine will not be visible to normal traceroute probes from hosts on your \"public\" interfaces.",
	'verbose_info' => "\"destination-unreachable\" is (ab)used by the traceroute program to check routing to individual hosts. ",
	},

	);

&InstallScript;

sub ipchainsShowHeading {
	# Display the header, centered
	my ( $header_text ) = @_;
	my ( $indent );
	$indent = int( (74 - length($header_text)) / 2);
	print "\n";
	while ( $indent-- > 0 ) { print " "; }
	print "-- $header_text --\n";
}

sub ipchainsAddNetmask {
	# adds netmask info to plain dotted-quad IP addresses
	my ( $phrase ) = @_;
	if ( $phrase =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ ) {
		# looks like a plain IPv4 dot-quad address
		if ( $phrase eq '0.0.0.0' ) {
			$phrase .= '/0.0.0.0';
		} elsif ( $phrase =~ /\.0\.0\.0$/ ) {
			$phrase .= '/255.0.0.0';
		} elsif ( $phrase =~ /\.0\.0$/ ) {
			$phrase .= '/255.255.0.0';
		} elsif ( $phrase =~ /\.0$/ ) {
			$phrase .= '/255.255.255.0';
		} else {
			$phrase .= '/255.255.255.255';
		}
	} 
	return $phrase;
}

sub ipchainsCleanItemList {
	# removes spaces, cleans up whitespace in string
	my ( $list ) = @_;
	my ( @items, $one );
	$list =~ s/\,/ /g;
	$list =~ s/[^a-zA-Z0-9\-\_\s\.\:\+\/]//eg;	# remove unexpected chars
	$list =~ s/[\n\r\t]/ /g;
	while ( $list =~ /  / ) { $list =~ s/  / /g;}
	@items = split(/ /, $list);
	$list = '';
	foreach $one ( @items ) {
		$list .= ' '.&ipchainsAddNetmask($one);
	}
	$list =~ s/^.//;
	return $list;
}

sub InstallScript {

   my ( @varArray, $position, $status, $mode, $model, $ok, $ans, $loop );

   my $insertPrefix = "\#\n\# Inserted by Bastille install script:\n";

   print <<ENDIPCHAINSQUERY;

IPCHAINS:

ipchains allows you to do packet filtering/modification via the Linux kernel.
You can use this to block certain types of connections from/to your machine,
to turn your machine into a small firewall, and to do Network Address
Translation, or "IP masquerading", which lets several machines share a single
IP address.

We have a particular script that will create firewalling instructions for you.
If we install the script, you will be prompted to make various choices (with
suggested defaults), but you may need to edit it for your particular site
and WILL need to individually activate it.

ENDIPCHAINSQUERY

   if (&GetYN("Would you like us to install the ipchains script? ") eq 'Y') {

print <<ENDOFINSTRUX;

You will be asked to choose initial settings for the firewall script. The 
defaults are generally the minimal recommended settings. To accept the 
default (shown in brackets), press RETURN. To change a non-empty default 
to an empty value, enter some whitespace before pressing RETURN.

Your responses should be whitespace delimited lists of items. IP addresses may 
be entered in plain "dotted-quad" notation, with or without netmasks.  For
instance,  "10.0.0.0/8" "10.0.0.0/255.0.0.0" "10.0.0.0"
will all be read as legitimate ways to express the 10.*.*.* "class A" network 
space.  If you have "unexpected" networks like "10.0.0.0/255.255.255.0" or 
"192.168.1.0/255.255.255.128", you will need to specify that explicitly.

Services can be entered as names ("smtp") or numbers ("25").  Be warned that
any names must explicitly match one of those listed in /etc/services. Ranges 
may be specified with colons, e.g. "1024:" indicates all ports >= 1024, 
"6000:6020" indicates ports 6000 to 6020, inclusive.

Unless you really understand networking, answer Y for the question:
ENDOFINSTRUX

	# how verbose the help should be
	$mode = 'spartan';
	#print "Would you more verbose instructions for each step?\n";
	if ( &GetYN("Would you prefer more verbose instructions for each step? ") eq 'Y' ) {
		$mode = 'verbose';
	}

	# whether to ask the complex multi-interface questions
	# (internal, IP Masq, etc)
	$model = 'basic';
	print <<ENDCOMPLEX;
Advanced networking options?

If this is a standalone workstation or server with a single network interface
(e.g. may connect to one of several PPP servers, but is never connected to two
different networks simultaneously), answer N.

If this is a server that deals with multiple interfaces or provides IP 
Masquerading/NAT service, answer Y.

ENDCOMPLEX

	if ( &GetYN("Advanced networking options? ") eq 'Y' ) {
		$model = 'server';
	}

      &B_open(*ORIG_FIREWALL,"firewall_ipchains.txt");

	# Ask the user about the different options, in order
	for ($loop = 0; $loop < scalar(@ipchainsOptions); ++$loop) {
		# in this we build our @varArray to hold the text to be inserted
		# in the /bin/sh script after the different stanzas
		if ( ($model eq 'basic') && ($ipchainsOptions[$loop]{type} ne 'basic') ) {
			# advanced option, don't even ask the user, just accept the default
			$varArray[$ipchainsOptions[$loop]{stanza}] .= 
			 "$ipchainsOptions[$loop]{varname}\=\"$ipchainsOptions[$loop]{default}\"\n";			
		} else {
			# display the "heading" property
			&ipchainsShowHeading( $ipchainsOptions[$loop]{heading} );
			# display the basic information, using the auto-wrap display function
			&B_Display( $ipchainsOptions[$loop]{basic_info} );
			# if the user chose to see more verbose information, give it to them	
			if ( $mode eq 'verbose' ) { 
				if ( $ipchainsOptions[$loop]{verbose_info} =~ /\S/ ) { 
					&B_Display( "\n".$ipchainsOptions[$loop]{verbose_info} ); 
				}
			}
			$ok = 0;
			print "\n";
			while ( $ok == 0 ) {
				# prompt the user, displaying the default value
				print "$ipchainsOptions[$loop]{varname} \[\"$ipchainsOptions[$loop]{default}\"\]: ";
				$ans = &GetString;
				$ans =~ s/[\r\n]//g;
				# replace any leading "&" chars with the default value
				# DISABLED $ans =~ s/^\s*?\&/$ipchainsOptions[$loop]{default} /;
				# if the user simply hit <return>, then use the default
				if ( $ans eq '' ) { $ans = $ipchainsOptions[$loop]{default}; }
				# clean up their input
				$ans = &ipchainsCleanItemList($ans);
				if ( ($ipchainsOptions[$loop]{varname} =~ /^TCP_.*?_SERVICES/) &&
					(($ans =~ /\bftp\b/) || ($ans =~ /\b21\b/)) ) {
					&B_Display("You have chosen to make FTP service available to remote clients. This will work fine for normal, \"active\" FTP clients, but may not be adequate for clients that attempt to use \"passive\" FTP mode. To enable passive FTP clients to use your FTP service, you will want to configure the passive port range and allow those ranges, too.\n");
				}
				# show the user what the var would be set to, ask their approval
				if ( &GetYN("$ipchainsOptions[$loop]{varname} will be set to \"$ans\". OK? ") eq 'Y' ) { $ok = 1; }
			}
			$varArray[$ipchainsOptions[$loop]{stanza}] .= 
				"$ipchainsOptions[$loop]{varname}\=\"$ans\"\n";				
		}
	}

      &B_open(*FIREWALL,">/etc/rc.d/init.d/bastille-firewall");
      #&B_open(*FIREWALL,">/tmp/bastille-firewall");

      # $status is used to note whether we should look for a blank
      # line to add variable settings from @varArray
      $status = 'clear';
      while ($line=<ORIG_FIREWALL>) {
	 if ( $line =~ /^\# ([0-9]{1,}?)\)/ ) {
		# Found a line beginning with "# nn)", the first line of
		# stanza "nn", so we should start looking for a blank
		# line to add our custom settings
		$position = $1;
		$status = 'wait';
	 }
	 if ( ($line =~ /^\s{0,}$/) && ($status eq 'wait') ) {
		# just passed a config stanza, add the variable-setting lines
         	&B_print(*FIREWALL,$insertPrefix.$varArray[$position]);
		# back to normal status
		$status = 'clear';
	 }
         &B_print(*FIREWALL,$line);
	 if ($line =~ /^#! \/bin\/sh/) {
	    &B_print(*FIREWALL,<<ENDCHKCFGADD);
#
# bastille-firewall      This shell script takes care of starting and stopping
#                        Bastille Linux's firewalling, via Peter W's script.
#
# chkconfig: 345 2 98
#
ENDCHKCFGADD
         }
      }

      &B_close(*FIREWALL);
      &B_close(*ORIG_FIREWALL);
      # There's no good reason for local users to read the firewall config,
      # is there?
      chmod 0500, '/etc/rc.d/init.d/bastille-firewall';
      
      print <<ENDEXPLFW;
      
The firewalling script is called /etc/rc.d/init.d/bastille-firewall.  Please
review its configuration for your site.  You can then test it by using

     /etc/rc.d/init.d/bastille-firewall start
and
     /etc/rc.d/init.d/bastille-firewall stop
     
Once you have a configuration that will work on your system, you can make it
run at every normal boot-up by typing 

    /sbin/chkconfig bastille-firewall on
      
ENDEXPLFW
   }
}

1;

